/*
    CONF - Config file support functions
    Copyright (C) 2002  by _Leo_

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "conf.h"

struct ConfigRecord {
	char *tag;
	char *value;
	void *next;
} cfg_pconfig;

/* Show configuration record */
void conf_show(void) {

	struct ConfigRecord *item;
	int last;
	
	item = &cfg_pconfig;

	while(item->next) {

		item = item->next;
		printf("conf: TAG: %s - VALUE: %s\n", item->tag, item->value);
	}
}

/* Parse confile file */
int conf_parse(char *filename) {

	FILE *file;
	char buffer[1024];
	int line = 1, exitcode = 1;

	/* Open file */
	if ((file = fopen(filename, "r")) != NULL) {

#ifdef DEBUG
	printf("conf_debug: Open file %s\n",filename);
#endif

		/* Entire file */
		while(!feof(file)) {

			if (fgets(buffer, sizeof(buffer), file)) {
				
				/* Parse this line */
				if (!conf_parse_line(buffer)) {
					exitcode = 0;
					fprintf(stderr, "conf: parse: error parsing line %d\n", line);
					break;
				}
				line++;
			}
		}

		fclose(file);

#ifdef DEBUG
	printf("conf_debug: Close file %s\n",filename);
#endif

	}
	else {
		perror("conf: parse");
		return 0;
	}

	return exitcode;
}

/* Parse a line */
int conf_parse_line(char *line) {

	char tag[512];
	char value[512];
	int i, j;

	/* NULL? */
	if (!line) return 0;

	/* Comments */
	if (line[0] == '#') return 1;

	/* Blank */
	if (line[0] == '\n') return 1;

	/* Get tag */
	i=0;
	j=0;
	while((line[i] != '=') && (line[i] != ' ') && (line[i] != '\0') && (line[i] != '\n')) tag[j++] = line[i++];
	tag[j] = '\0';

	while((line[i] == '=') || (line[i] == ' ')) i++;

	/* Get value */
	j=0;
	while((line[i] != '\0') && (line[i] != '\n')) value[j++] = line[i++];
	value[j] = '\0';

#ifdef DEBUG
	printf("conf_debug: TAG: %s - VALUE: %s\n", tag, value);
#endif

	/* Store */
	if ((tag[0]) && (value[0])) {
		return (conf_store(tag, value));
	}
	else {
		return 0;
	}

	return 1;
}

/* Store config record*/
int conf_store(char *tag, char *value) {

	struct ConfigRecord *item;

	/* Get last record */
	item = &cfg_pconfig;
	while(item->next) item = item->next;


	/* Allocate mem for cfg record */
	item->next = (void *)malloc(sizeof(cfg_pconfig));
	item = item->next;

	if (item) {
		/* Save tag */
		item->tag = (void *)malloc(strlen(tag)+1);
		strcpy(item->tag, tag);

		/* Save value */
		item->value = (void *)malloc(strlen(value)+1);
		strcpy(item->value, value);

		/* End */
		item->next = NULL; 
	}
	else {
		fprintf(stderr, "conf: can't allocate memory [%d bytes]", sizeof(cfg_pconfig));
		return 0;
	}

	return 1;
}

/* Query value for given tag */
int conf_query(const char *tag, char *value) {

	struct ConfigRecord *item;
	int last=1;
	
	item = cfg_pconfig.next;

	if (item->tag) {
		while((last=strcmp(item->tag, tag)) && (item->next)) {
			item = item->next;
		}
	}
		
	if (last == 0) {
		strcpy(value, item->value);
	}	
	else {
		return 0;
	}

	return 1;
}
