/*
    WatchMan - Process killer daemon
    Copyright (C) 2002  by _Leo_

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "main.h"

/*=== MAIN ===*/

int main(int argc, char *argv[]) {
	
	struct dirent **process;
	int count, i, pid, time, retval = 0, daemon = 0;
	char exe[WM_MAXFILENAME];
	char str[512];

	int cfg_maxtime;
	char cfg_procname[256];
	int cfg_killsignal = WM_KSIGNAL;
	char cfg_procdir[256]={WM_PROC};

	if (argv[1]) {
		if (!strcmp(argv[1], "--daemon")) {
			if ((i = fork()) < 0) {                                                                                              
				perror("fork()");                                                                                                
				return 1;                                                                                                        
			}
			else {
				if (i != 0) {                                                                                                 
					exit(0);                                                                                                         
				}                                                                                                                    
				printf("Watchman daemon started\n\n");
				daemon = 1;
			}
		}
	}


	/*=== Read configuration file ====*/

	if (conf_parse(WM_CONFFILE)) {

		if (conf_query("ProcessTimeOut", str)) {
			cfg_maxtime = atoi(str);
			
			if ((cfg_maxtime<1) || (cfg_maxtime>32767)) {
				fprintf(stderr, "watchman: configuration ProcessTimeOut should be between 1 and 32767\n");
				retval = 1;
			}
		}
		else {
			fprintf(stderr, "watchman: configuration ProcessTimeOut is mandatory\n");
			retval = 1;
		}

		if (conf_query("ProcessName", cfg_procname)) {

			if ((strlen(cfg_procname)<3) || (strlen(cfg_procname)>256)) {
				fprintf(stderr, "watchman: configuration ProcessName should be between 3 and 256 characters long\n");
				retval = 1;
			}
		}
		else {
			fprintf(stderr, "watchman: configuration ProcessName is mandatory\n");
			retval = 1;
		}

		if (conf_query("KillSignal", str)) {
			cfg_killsignal = atoi(str);
			
			if ((cfg_killsignal<1) || (cfg_killsignal>32767)) {
				fprintf(stderr, "watchman: configuration KillSignal should be between 1 and 32767\n");
				retval = 1;
			}
		}

		if (conf_query("ProcDir", cfg_procdir)) {

			if ((strlen(cfg_procdir)<3) || (strlen(cfg_procdir)>256)) {
				fprintf(stderr, "watchman: configuration ProcDir should be between 3 and 256 characters long\n");
				retval = 1;
			}
		}

		if (!retval) {
			/*=== Look for bad processes ===*/

			/* Main loop */
			do {

				if (daemon) sleep(30);

				/* Scan /proc, look for number directories */
				scan(cfg_procdir, &process, &count);

				/* Get info for each process */
				for(i=0; i<count; i++) {
					time = gettime(process[i]->d_name, exe, &pid);

					/* If TIME > cfg_maxtime and it is target process, kill it */
					if ((time > cfg_maxtime) && (!strcmp(exe, cfg_procname))){
						kill(pid, cfg_killsignal); 
						//printf("should kill: %d\n", pid);
					}
			
					free(process[i]);
					
				}

				free(process);
			}
			while(daemon);
		}
	}
	else {
		fprintf(stderr, "watchman: error parsing configuration file %s", WM_CONFFILE);
	}

	return retval;	
}

/* Get process info for dir */
int gettime(char *dir, char *strexe, int *pid) {

	char filename[WM_MAXFILENAME];
	char buffer[WM_MAXLINE];
	FILE *f;
	int utime, stime, i, pos;
	char state;
	char exe[WM_MAXFILENAME];

	/* Build filename string */
	sprintf(filename, "%s%s/stat\0", WM_PROC, dir);

	/* Open file */
	if (f=fopen(filename, "r")) {

		/* Get content */	
		fread(buffer, sizeof(buffer), 1, f);

		/* Parse */
		scanline(buffer, pid, strexe, &state, &utime, &stime);

		fclose(f);
	}

	return ((utime + stime)/100);

}

/* Fill namelist with process numbers */
void scan(const char *dir, struct dirent ***namelist, int *entries) {

	/* Get array of process numbers */
	*entries = scandir(dir, namelist, matchint, 0);

}

/* Selects number named directory entries */
int matchint(const struct dirent *dir) {

	if (atoi(dir->d_name)>0) 
		return 1;
	else
		return 0;

}

